/**
	$Id $

	Copyright (C) 2011 Nicklas Nordborg

	This file is part of BASE - BioArray Software Environment.
	Available at http://base.thep.lu.se/

	BASE is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 3
	of the License, or (at your option) any later version.

	BASE is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with BASE. If not, see <http://www.gnu.org/licenses/>.
*/
package net.sf.basedb.clients.web.extensions;

import javax.servlet.http.HttpServletRequest;

import net.sf.basedb.util.Values;
import net.sf.basedb.util.extensions.manager.ExtensionsFile;
import net.sf.basedb.util.filter.Filter;

/**
	Filter implementation that selects only those extensions that
	was selected by the user in the gui. Eg. if the request
	parameter <i>filename</i>.install is set.
	@author Nicklas
	@since 3.0
	@base.modified $Date $
*/
public class InstallFilter 
	implements Filter<ExtensionsFile>
{

	private final HttpServletRequest request;
	
	public InstallFilter(HttpServletRequest request)
	{
		this.request = request;
	}
	
	@Override
	public boolean evaluate(ExtensionsFile xtFile) 
	{
		if (!xtFile.isValid() || (xtFile.hasError())) 
		{
			// Do not install files with errors
			return false;
		}
		
		return Values.getBoolean(request.getParameter(xtFile.getName() + ".install"));
	}

}
