/**
	$Id: SetFileStatusProcessor.java 5605 2011-04-12 12:46:47Z nicklas $

	Copyright (C) 2011 Nicklas Nordborg

	This file is part of BASE - BioArray Software Environment.
	Available at http://base.thep.lu.se/

	BASE is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 3
	of the License, or (at your option) any later version.

	BASE is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with BASE. If not, see <http://www.gnu.org/licenses/>.
*/
package net.sf.basedb.clients.web.extensions;

import net.sf.basedb.util.extensions.manager.ExtensionsFile;
import net.sf.basedb.util.extensions.manager.ExtensionsFileProcessor;
import net.sf.basedb.util.extensions.manager.ExtensionsManager;
import net.sf.basedb.util.extensions.manager.ProcessResults;
import net.sf.basedb.util.extensions.manager.ExtensionsFile.WriteableExtensionsFile;
import net.sf.basedb.util.extensions.manager.Settings;

/**
	File processor implementation that finalizes the uninstallation
	of an extensions file by removing it from the settings.
	See {@link Settings#setUninstalledFile(java.io.File)} and
	{@link WriteableExtensionsFile#setUninstalled()}

	@author Nicklas
	@since 3.0
	@base.modified $Date: 2011-04-12 14:46:47 +0200 (ti, 12 apr 2011) $
*/
public class UninstalledFileProcessor
	implements ExtensionsFileProcessor
{

	private final ProcessResults results;
	private int numFiles;

	public UninstalledFileProcessor(ProcessResults results)
	{
		this.results = results;
	}
	
	/*
		From the RegisterExtensionsProcessor class
		------------------------------------------
	*/
	@Override
	public void begin(ExtensionsManager manager, int numFiles)
	{
		this.numFiles = 0;
	}

	@Override
	public void processFile(ExtensionsManager manager, WriteableExtensionsFile wFile)
	{
		ExtensionsFile xtFile = wFile.getExtensionsFile();
		results.setStatus(xtFile, "Uninstalled");
		wFile.setUninstalled();
		numFiles++;
	}
	@Override
	public void done(ExtensionsManager manager)
	{}

	@Override
	public void done(ExtensionsManager manager, Throwable t)
	{}
	// ---------------------------------------------

	/**
		Get the number of files that was processed.
	*/
	public int getNumFiles()
	{
		return numFiles;
	}


}
