/**
	$Id:CompactButtonRenderer.java 4187 2008-03-20 11:15:25Z nicklas $

	Copyright (C) Authors contributing to this file.

	This file is part of BASE - BioArray Software Environment.
	Available at http://base.thep.lu.se/

	BASE is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 3
	of the License, or (at your option) any later version.

	BASE is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with BASE. If not, see <http://www.gnu.org/licenses/>.
*/
package net.sf.basedb.clients.web.extensions.toolbar;

import java.io.IOException;
import java.io.Writer;

import net.sf.basedb.clients.web.extensions.JspContext;
import net.sf.basedb.core.SessionControl;
import net.sf.basedb.util.Values;
import net.sf.basedb.util.extensions.Renderer;

/**
	A renderer for toolbar button actions, which displays the 
	action as a toolbar button. The render can also be used in
	"standalone" mode (eg. without a {@link JspContext}).

	@author nicklas
	@version 2.12
	@base.modified $Date:2008-03-20 12:15:25 +0100 (Thu, 20 Mar 2008) $
*/
public class ToolbarButtonRenderer
	implements Renderer<ButtonAction>
{

	private JspContext context;
	
	/**
		Create a new "standalone" toolbar button renderer.
		Use {@link #getHTML(ButtonAction, boolean, boolean)} to
		generate the HTML. Do not use {@link #render(ButtonAction)} since
		that requires a {@link JspContext}.
	*/
	public ToolbarButtonRenderer()
	{}
	
	/**
		Create a new toolbar button renderer.
	*/
	public ToolbarButtonRenderer(JspContext context)
	{
		this.context = context;
	}
	
	/*
		From the Renderer interface
		---------------------------
	*/
	/**
		Generates a toolbar button. If the button isn't visible nothing is 
		generated. If the button isn't enabled, the onclick attribute is not 
		generated.
	*/
	@Override
	public void render(ButtonAction btn)
	{
		if (!btn.isVisible()) return;
		Writer out = context.getOut();
		SessionControl sc = context.getSessionControl();
		try
		{
			boolean hasText = true;
			boolean hasImages = true;
			if (sc != null)
			{
				hasImages = Values.getBoolean(sc.getUserClientSetting("toolbar.images"), true);
				hasText = Values.getBoolean(sc.getUserClientSetting("toolbar.text"), true);
			}
			out.write(getHTML(btn, hasText, hasImages));
		}
		catch (IOException ex)
		{
			throw new RuntimeException(ex);
		}
	}
	// ----------------------------------------------

	/**
		Generate the HTML that is needed to create a toolbar button.
		@param btn The button information
		@param displayText If text should be displayed on the button or not
		@param displayIcon If the icon should be displayed on the button or not
	*/
	public String getHTML(ButtonAction btn, boolean displayText, boolean displayIcon)
	{
		StringBuilder sb = new StringBuilder();
		sb.append("<div");
		if (btn.getId() != null) sb.append(" id=\"").append(btn.getId()).append("\"");
		sb.append(" class=\"").append(btn.getClazz() == null ? "button" : btn.getClazz()).append("\"");
		if (btn.getStyle() != null) sb.append(" style=\"").append(btn.getStyle()).append("\"");

		if (btn.isEnabled() && btn.getOnClick() != null) 
		{
			sb.append(" onclick=\"").append(btn.getOnClick()).append("\"");
		}
		String tooltip = btn.getTooltip();
		if (!displayText && tooltip == null) tooltip = btn.getTitle();			
		if (tooltip != null) sb.append(" title=\"").append(tooltip).append("\"");
		sb.append(">");
		if (displayIcon && btn.getIcon() != null)
		{
			sb.append("<table><tr>");
			sb.append("<td><img src=\"").append(btn.getIcon()).append("\"");
			if (btn.getId() != null) sb.append(" id=\"").append(btn.getId()).append(".icon").append("\"");
			sb.append(" onerror=\"this.style.display='none'\"></td>");
			sb.append("<td>");
		}
		if (displayText && btn.getTitle() != null) 
		{
			sb.append("<span style=\"padding: 0px 2px 0px 2px;\"");
			if (btn.getId() != null) sb.append(" id=\"").append(btn.getId()).append(".text").append("\"");
			sb.append(">").append(btn.getTitle()).append("</span>");
		}
		if (displayIcon && btn.getIcon() != null)
		{
			sb.append("</td>");
			sb.append("</tr></table>");
		}
		sb.append("</div>");
		return sb.toString();
	}
	
}
