/**
	$Id: FileServerData.java 5818 2011-10-21 11:15:41Z nicklas $

	Copyright (C) 2010 Nicklas Nordborg

	This file is part of BASE - BioArray Software Environment.
	Available at http://base.thep.lu.se/

	BASE is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 3
	of the License, or (at your option) any later version.

	BASE is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with BASE. If not, see <http://www.gnu.org/licenses/>.
*/
package net.sf.basedb.core.data;

import java.util.Set;

/**
	File entries that are external files can be linked to a file server. This
	contains information about username/password and/or SSL keys that are
	required to access the server. 

	@author Nicklas
	@since 2.16
	@base.modified $Date: 2011-10-21 13:15:41 +0200 (Fri, 21 Oct 2011) $
	@hibernate.class table="`FileServers`" lazy="true"
	@see <a href="../../../../../../html/developer/api/data_api.files.html">Developer documentation: Files and directories</a>
*/
public class FileServerData
	extends CommonData
{

	public FileServerData()
	{}
	
	public static final int MAX_CONNECTION_MANAGER_FACTORY_LENGTH = 255;
	private String connectionManagerFactory;
	/**
		Get the ID of the connection manager factory that should be
		used to retreieve the file contents and metadata. If null 
		auto-detection is used to find a suitable connection manager 
		factory.
		@since 3.0
		@hibernate.property column="`cm_factory`" type="string" length="255" not-null="false"
	*/
	public String getConnectionManagerFactory()
	{
		return connectionManagerFactory;
	}
	public void setConnectionManagerFactory(String connectionManagerFactory)
	{
		this.connectionManagerFactory = connectionManagerFactory;
	}
	
	public static final int MAX_HOST_LENGTH = 255;
	private String host;
	/**
		Get the name:port of the actual host to use for all files
		using this server (overrides the host specified in the file url)
		@since 3.0
		@hibernate.property column="`host`" type="string" length="255" not-null="false"
	*/
	public String getHost()
	{
		return host;
	}
	public void setHost(String host)
	{
		this.host = host;
	}
	
	public static final int MAX_USERNAME_LENGTH = 255;
	private String username;
	/**
		Get the username that is used to login to the server. If null
		no login is required.
		@hibernate.property column="`username`" type="string" length="255" not-null="false"
	*/
	public String getUsername()
	{
		return username;
	}
	public void setUsername(String username)
	{
		this.username = username;
	}

	public static final int MAX_PASSWORD_LENGTH = 255;
	private String password;
	/**
		Get the password that is used to login to the server. 
		@hibernate.property column="`password`" type="string" length="255" not-null="false"
	*/
	public String getPassword()
	{
		return password;
	}
	public void setPassword(String password)
	{
		this.password = password;
	}
	
	public static final int MAX_CERTIFICATE_LENGTH = 65535;
	private byte[] serverCertificate;
	/**
		Get the public server certificate indicating that we should trust
		https connections to the server.
		@hibernate.property column="`server_certificate`" type="binary" length="65535" not-null="false"
	*/
	public byte[] getServerCertificate()
	{
		return serverCertificate;
	}
	public void setServerCertificate(byte[] serverCertificate)
	{
		this.serverCertificate = serverCertificate;
	}
	
	private byte[] clientCertificate;
	/**
		Get the client certificate that we need to send to the server 
		to make the server accept our connection.
		@hibernate.property column="`client_certificate`" type="binary" length="65535" not-null="false"
	*/
	public byte[] getClientCertificate()
	{
		return clientCertificate;
	}
	public void setClientCertificate(byte[] clientCertificate)
	{
		this.clientCertificate = clientCertificate;
	}
	
	private String clientPassword;
	/**
		Get the password that is used to unlock the client certificate.
		@hibernate.property column="`client_password`" type="string" length="255" not-null="false"
	*/
	public String getClientCertificatePassword()
	{
		return clientPassword;
	}
	public void setClientCertificatePassword(String clientPassword)
	{
		this.clientPassword = clientPassword;
	}
	
	private Set<FileData> files;
	/**
		This is the inverse end.
		@see FileData#getFileServer()
		@hibernate.set lazy="true" inverse="true"
		@hibernate.collection-key column="`fileserver_id`"
		@hibernate.collection-one-to-many class="net.sf.basedb.core.data.FileData"
	*/
	Set<FileData> getFiles()
	{
		return files;
	}
	void setFiles(Set<FileData> files)
	{
		this.files = files;
	}
	
}
