/*
  $Id $

	Copyright (C) 2011 Nicklas Nordborg

	This file is part of BASE - BioArray Software Environment.
	Available at http://base.thep.lu.se/

	BASE is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 3
	of the License, or (at your option) any later version.
	
	BASE is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.
	
	You should have received a copy of the GNU General Public License
	along with BASE. If not, see <http://www.gnu.org/licenses/>.
*/
package net.sf.basedb.core.data;

import java.util.Date;
import java.util.HashMap;
import java.util.Map;

/**
	A template for holding information about reporter properties that are/should
	be cloned in the dynamic database. Unlocked templates are created by
	a admin/power user and can be used in a cloning operation. The cloning
	operation creates a locked copy of the template that is attached to the
	{@link VirtualDbData} item.
	
	@author Nicklas
	@since 3.1
	@see net.sf.basedb.core.ReporterCloneTemplate
	@hibernate.class table="`ReporterCloneTemplates`" lazy="true"
	@base.modified $Date $
	@see <a href="../../../../../../html/developer/api/data_api.reporters.html">Developer documentation: Reporters</a>
*/
public class ReporterCloneTemplateData
	extends CommonData
	implements RegisteredData, LoggableData
{
	public ReporterCloneTemplateData()
	{}

	/*
		From the RegisteredData interface
		-------------------------------------------
	*/
	private Date entryDate;
	public Date getEntryDate()
	{
		return entryDate;
	}
	public void setEntryDate(Date entryDate)
	{
		this.entryDate = entryDate;
	}
	// -------------------------------------------

	
	private boolean locked;
	/**
		The template is locked when it is used by a {@link VirtualDbData}. The
		locked status can never be changed.
		@hibernate.property column="`locked`" type="boolean" not-null="true" update="false"
	*/
	public boolean isLocked()
	{
		return locked;
	}
	public void setLocked(boolean locked)
	{
		this.locked = locked;
	}

	private long bytes;
	/**
		The number of bytes the cloned reporter information uses in the 
		dynamic database.
		@hibernate.property column="`bytes`" type="long" not-null="true"
	*/
	public long getBytes()
	{
		return bytes;
	}
	public void setBytes(long bytes)
	{
		this.bytes = bytes;
	}

	private Map<String, ReporterClonePropertyData> properties;
	/**
		Get the map that manages cloned properties.
		
		@hibernate.map table="`ReporterCloneProperties`" lazy="true" 
		@hibernate.collection-key column="`template_id`"
		@hibernate.collection-index column="`property_name`" type="string" length="255" 
		@hibernate.collection-composite-element class="net.sf.basedb.core.data.ReporterClonePropertyData" not-null="true"
	*/
	public Map<String, ReporterClonePropertyData> getProperties()
	{
		if (properties == null)
		{
			properties = new HashMap<String, ReporterClonePropertyData>();
		}
		return properties;
	}
	void setProperties(Map<String, ReporterClonePropertyData> properties)
	{
		this.properties = properties;
	}

}
