/**
	$Id: TransactionDetails.java 5038 2009-08-07 10:50:01Z nicklas $

	Copyright (C) 2009 Nicklas Nordborg

	This file is part of BASE - BioArray Software Environment.
	Available at http://base.thep.lu.se/

	BASE is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 3
	of the License, or (at your option) any later version.

	BASE is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with BASE. If not, see <http://www.gnu.org/licenses/>.
*/
package net.sf.basedb.core.log;

import java.util.Date;

import net.sf.basedb.core.SessionControl;
import net.sf.basedb.core.data.ChangeHistoryData;

/**
	Holds information about the curren transaction, such as the logged in user,
	the active project, the plug-in that is executing, etc.

	@author Nicklas
	@version 2.13
	@base.modified $Date: 2009-08-07 12:50:01 +0200 (Fri, 07 Aug 2009) $
*/
public class TransactionDetails
{
	private final int userId;
	private final int sessionId;
	private final int clientId;
	private final int projectId;
	private final int pluginId;
	private final int jobId;
	
	/**
		Creates a new transaction details object with information from 
		given session control
	*/
	public TransactionDetails(SessionControl sc)
	{
		this.userId = sc.getLoggedInUserId();
		this.sessionId = sc.getCurrentSessionId();
		this.clientId = sc.getClientId();
		this.projectId = sc.getActiveProjectId();
		this.jobId = sc.getJobId();
		this.pluginId = sc.getPluginId();
	}
	
	/**
		Get the ID of the currently logged in user, or 0 if no user is logged in.
	*/
	public int getUserId()
	{
		return userId;
	}
	
	/**
		Get the ID of the session or 0 if no user is logged in.
	*/
	public int getSessionId()
	{
		return sessionId;
	}

	/**
		Get the ID of the client appliction that is used, or 0 if not known.
	*/
	public int getClientId()
	{
		return clientId;
	}
	
	/**
		Get the ID of the currently active project, or 0 if no project is active.
	*/
	public int getProjectId()
	{
		return projectId;
	}
	
	/**
		Get the ID of the currently running plug-in or 0 if no plug-in is running.
	*/
	public int getPluginId()
	{
		return pluginId;
	}
	
	/**
		Get the ID of the currently running job or 0 if no job is running.
	*/
	public int getJobId()
	{
		return jobId;
	}

	/**
		Convert the information about the transaction to a
		{@link ChangeHistoryData} object.
		@return A ChangeHistoryData object
	*/
	public ChangeHistoryData toChangeHistoryData()
	{
		ChangeHistoryData history = new ChangeHistoryData();
		history.setTime(new Date(System.currentTimeMillis()));
		history.setUserId(userId);
		history.setSessionId(sessionId);
		history.setClientId(clientId == 0 ? null : clientId);
		history.setProjectId(projectId == 0 ? null : projectId);
		history.setPluginId(pluginId == 0 ? null : pluginId);
		history.setJobId(jobId == 0 ? null : jobId);
		return history;
	}
	
	
}
