/**
	$Id: BioMaterialEventLogger.java 5642 2011-05-26 07:37:32Z nicklas $

	Copyright (C) 2009 Nicklas Nordborg

	This file is part of BASE - BioArray Software Environment.
	Available at http://base.thep.lu.se/

	BASE is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 3
	of the License, or (at your option) any later version.

	BASE is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with BASE. If not, see <http://www.gnu.org/licenses/>.
*/
package net.sf.basedb.core.log.db;

import net.sf.basedb.core.BioMaterialEvent;
import net.sf.basedb.core.data.BioMaterialEventData;
import net.sf.basedb.core.data.ChangeHistoryDetailData;
import net.sf.basedb.core.log.EntityDetails;
import net.sf.basedb.core.log.ChangeType;
import net.sf.basedb.core.log.EntityLogger;
import net.sf.basedb.core.log.LogManager;

/**
	Logger for biomaterial events that logs UPDATE:s to 
	CREATION and HYBRIDIZATION events as updates to their
	parent biomaterial/hybridization. OTHER events are not 
	logged.

	@author Nicklas
	@version 2.13
	@base.modified $Date: 2011-05-26 09:37:32 +0200 (Thu, 26 May 2011) $
*/
public class BioMaterialEventLogger
	implements EntityLogger
{

	private final boolean detailedProperties;

	/**
		Creates a new biomaterial event logger.
		@param detailedProperties If TRUE, the log includes a list with
			the properties that was modified
	*/
	public BioMaterialEventLogger(boolean detailedProperties)
	{
		this.detailedProperties = detailedProperties;
	}
	
	/*
		From the EntityLogger interface
		-------------------------------
	*/
	@Override
	public void logChanges(LogManager logManager, EntityDetails details)
	{
		BioMaterialEventData event = (BioMaterialEventData)details.getEntity();
		ChangeHistoryDetailData change = null;
		ChangeType changeType = details.getChangeType();
		BioMaterialEvent.Type eventType = BioMaterialEvent.Type.fromValue(event.getEventType());
		
		if (changeType == ChangeType.UPDATE)
		{
			if (eventType == BioMaterialEvent.Type.CREATION)
			{
				change = details.toChangeHistoryDetailData(event.getBioMaterial(), ChangeType.UPDATE, detailedProperties);
			}
			else if (eventType == BioMaterialEvent.Type.BIOASSAY)
			{
				change = details.toChangeHistoryDetailData(event.getPhysicalBioAssay(), ChangeType.UPDATE, detailedProperties);
			}
		}
		if (change != null) ((DbLogManager)logManager).logChangeDetails(change);
	}
	// -------------------------------
}
