/**
	$Id: GenericExtraFileImporter.java 5238 2010-02-09 10:45:56Z nicklas $

	Copyright (C) 2010 Nicklas Nordborg

	This file is part of BASE - BioArray Software Environment.
	Available at http://base.thep.lu.se/

	BASE is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 3
	of the License, or (at your option) any later version.

	BASE is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with BASE. If not, see <http://www.gnu.org/licenses/>.
*/
package net.sf.basedb.util.bfs;

import java.io.IOException;
import java.io.InputStream;

import net.sf.basedb.core.DbControl;
import net.sf.basedb.core.Directory;
import net.sf.basedb.core.File;
import net.sf.basedb.core.ItemNotFoundException;
import net.sf.basedb.util.FileUtil;

/**
	Extra file importer implementation that imports file to a specified
	directory in the BASE file system. Files that already exists can
	be overwritten or ignored. Files that already exists and have their
	removed flag set are always overwritten.

	@author Nicklas
	@since 2.15
	@base.modified $Date: 2010-02-09 11:45:56 +0100 (Tue, 09 Feb 2010) $
*/
public class GenericExtraFileImporter
	implements ExtraFileImporter
{

	private final DbControl dc;
	private final Directory dir;
	private final boolean overwrite;

	/**
		Create a new extra file importer.
		@param dc A DbControl to use for database access
		@param dir The directory in the BASE file system where the imported files
			should be placed
		@param overwrite TRUE to force overwriting existing files, FALSE to only
			overwrite files that have been marked for removal
	*/
	public GenericExtraFileImporter(DbControl dc, Directory dir, boolean overwrite)
	{
		this.dc = dc;
		this.dir = dir;
		this.overwrite = overwrite;
	}
	
	@Override
	public File importExtraFile(String filename, InputStreamLocator locator)
		throws IOException
	{
		File f = null;
		try
		{
			f = File.getFile(dc, dir, filename, true);
			if (!f.isInDatabase())
			{
				dc.saveItem(f);
			}
			if (overwrite || !f.isInDatabase() || f.isRemoved())
			{
				InputStream in = null;
				try
				{
					in = locator.getInputStream(filename);
					FileUtil.copy(in, f.getUploadStream(false));
					f.setRemoved(false);
				}
				finally
				{
					FileUtil.close(in);
				}
			}
		}
		catch (ItemNotFoundException ex)
		{}
		return f;
	}

}
