/**
	$Id: GtfValidationFactory.java 5764 2011-09-27 12:35:28Z nicklas $

	Copyright (C) 2011 Nicklas Nordborg

	This file is part of BASE - BioArray Software Environment.
	Available at http://base.thep.lu.se/

	BASE is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 3
	of the License, or (at your option) any later version.

	BASE is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with BASE. If not, see <http://www.gnu.org/licenses/>.
*/
package net.sf.basedb.util.gtf;

import net.sf.basedb.core.ArrayDesign;
import net.sf.basedb.core.DataFileType;
import net.sf.basedb.core.DbControl;
import net.sf.basedb.core.Platform;
import net.sf.basedb.util.extensions.ActionFactory;
import net.sf.basedb.util.extensions.InvokationContext;

/**
	Action factory for creating GTF file validators.
	
	@author Nicklas
	@since 3.0
	@base.modified $Date: 2011-09-27 14:35:28 +0200 (Tue, 27 Sep 2011) $
*/
public class GtfValidationFactory
	implements ActionFactory<GtfValidationAction>
{

	public GtfValidationFactory()
	{}
	
	/*
		From the ActionFactory interface
		--------------------------------
	*/
	@Override
	public boolean prepareContext(InvokationContext<? super GtfValidationAction> context)
	{
		Object item = context.getClientContext().getCurrentItem();
		boolean isInContext = false;
		if (item instanceof ArrayDesign)
		{
			// Validation context
			ArrayDesign design = (ArrayDesign)item;
			isInContext = design.isPlatform(Platform.SEQUENCING);
		}
		else if (item instanceof DataFileType)
		{
			// Do we support validating the given file type?
			DataFileType fileType = (DataFileType)item;
			isInContext = DataFileType.REF_SEQ_GTF.equals(fileType.getExternalId());
		}
		return isInContext;
	}


	@Override
	public GtfValidationAction[] getActions(InvokationContext<? super GtfValidationAction> context)
	{
		Object item = context.getClientContext().getCurrentItem();
		ArrayDesign ownerItem = null;
		if (item instanceof ArrayDesign)
		{
			ownerItem = (ArrayDesign)item;
		}
		
		GtfValidationAction[] actions = null;
		if (ownerItem != null)
		{
			DbControl dc = context.getClientContext().getDbControl();
			actions = new GtfValidationAction[] { new GtfValidationAction(dc, ownerItem, Integer.MAX_VALUE) };
		}
		return actions;
	}
	// -----------------------------------

}
