/*
	$Id: TestDerivedBioAssay.java 5748 2011-09-19 10:50:09Z nicklas $

	Copyright (C) 2005 Nicklas Nordborg
	Copyright (C) 2006 Jari Häkkinen, Nicklas Nordborg

	This file is part of BASE - BioArray Software Environment.
	Available at http://base.thep.lu.se/

	BASE is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 3
	of the License, or (at your option) any later version.

	BASE is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with BASE. If not, see <http://www.gnu.org/licenses/>.
*/
import net.sf.basedb.core.*;

import java.util.Date;
import java.util.Set;

public class TestDerivedBioAssay
{

	static boolean ok = true;
	public static void main(String[] args)
	{
		TestUtil.checkArgs(args);
		TestUtil.begin();
		ok = test_all();
		TestUtil.stop();
	}

	static boolean test_all()
	{
		write("++Testing derived bioassays");
		write_header();
		
		int extractId1 = TestExtract.test_create(0, "Extract #1", 0, 0, false);
		int extractId2 = TestExtract.test_create(0, "Extract #2", 0, 0, false);
		
		int hybridizationId = TestPhysicalBioAssay.test_create(0, "Test hyb", 0, 2, extractId1, extractId2);
		int scannerId = TestHardware.test_create(SystemItems.getId(Hardware.SCANNER), null, false);
		int protocolId = TestProtocol.test_create(SystemItems.getId(Protocol.SCANNING), null, false);
		
		// Standard tests: create, load, list
		int id = test_create_root("Scan #1", hybridizationId, SystemItems.getId(DerivedBioAssay.SCAN), protocolId, scannerId, 0);
		int id2 = test_create_root("Scan #2", hybridizationId, 0, 0, 0, 0);
		
		// Children and bioassays
		int child1 = test_create_child("Child #1", id, 0, 0, 0, 0, 0);
		int child2 = test_create_child("Child #2", id, 0, 0, 0, 0, 0);

		// Listing
		test_load(id);
		test_list(hybridizationId, 2);
		test_list_children(id, 2);
		
		// Adding files
		int samId = TestFile.test_create("data/test.assembly.sam", false, false);
		TestDataFileType.test_set_file(Item.DERIVEDBIOASSAY, child1, DataFileType.SAM, samId);
		
		// Standard test: Delete
		if (TestUtil.waitBeforeDelete()) TestUtil.waitForEnter();
		test_delete(child1);
		test_delete(child2);
		test_delete(id);
		test_delete(id2);
		
		TestFile.test_delete(samId);
		TestPhysicalBioAssay.test_delete(hybridizationId);
		TestExtract.test_delete(extractId2);
		TestExtract.test_delete(extractId1);
		TestHardware.test_delete(scannerId);
		TestProtocol.test_delete(protocolId);
		write("++Testing derived bioassays "+(ok ? "OK" : "Failed")+"\n");
		return ok;
	}

	static int test_create_root(String name, int bioAssayId, int subtypeId, int protocolId, int hardwareId, int softwareId)
	{
		if (bioAssayId == 0 || !TestUtil.hasPermission(Permission.CREATE, Item.DERIVEDBIOASSAY)) return 0;
		int id = 0;
		DbControl dc = null;
		try
		{
			dc = TestUtil.getDbControl();
			DerivedBioAssay dbas = DerivedBioAssay.getNew(dc, PhysicalBioAssay.getById(dc, bioAssayId), null);
			dbas.setName(name);
			dbas.setDescription("Added at "+new Date());
			if (subtypeId != 0)
			{
				dbas.setItemSubtype(ItemSubtype.getById(dc, subtypeId));
			}
			if (hardwareId != 0)
			{
				dbas.setHardware(Hardware.getById(dc, hardwareId));
			}
			if (softwareId != 0)
			{
				dbas.setSoftware(Software.getById(dc, softwareId));
			}
			if (protocolId != 0)
			{
				dbas.setProtocol(Protocol.getById(dc, protocolId));
			}
			dc.saveItem(dbas);
			dc.commit();
			id = dbas.getId();
			dc = TestUtil.getDbControl();
			dc.reattachItem(dbas, false);
			write_item(0, dbas);
			write("--Create root derived bioassay OK");
		}
		catch (Throwable ex)
		{
			write("--Create root derived bioassay FAILED");
			ex.printStackTrace();
			ok = false;
		}
		finally
		{
			if (dc != null) dc.close();
		}
		return id;
	}

	static int test_create_child(String name, int parentId, int extractId, int subtypeId, 
			int protocolId, int hardwareId, int softwareId)
	{
		if (parentId == 0) return 0;
		int id = 0;
		DbControl dc = null;
		try
		{
			dc = TestUtil.getDbControl();
			DerivedBioAssay dbas = DerivedBioAssay.getNew(dc, DerivedBioAssay.getById(dc, parentId), null);
			dbas.setName(name);
			dbas.setDescription("Added at "+new Date());
			if (extractId != 0)
			{
				dbas.setExtract(Extract.getById(dc, extractId));
			}
			if (subtypeId != 0)
			{
				dbas.setItemSubtype(ItemSubtype.getById(dc, subtypeId));
			}
			if (hardwareId != 0)
			{
				dbas.setHardware(Hardware.getById(dc, hardwareId));
			}
			if (softwareId != 0)
			{
				dbas.setSoftware(Software.getById(dc, softwareId));
			}
			if (protocolId != 0)
			{
				dbas.setProtocol(Protocol.getById(dc, protocolId));
			}
			dc.saveItem(dbas);
			dc.commit();
			id = dbas.getId();
			dc = TestUtil.getDbControl();
			dc.reattachItem(dbas, false);
			write_item(0, dbas);
			write("--Create child derived bioassay OK");
		}
		catch (Throwable ex)
		{
			write("--Create child derived bioassay FAILED");
			ex.printStackTrace();
			ok = false;
		}
		finally
		{
			if (dc != null) dc.close();
		}
		return id;
	}

	
	static void test_load(int id)
	{
		if (id == 0) return;
		DbControl dc = null;
		try
		{
			dc = TestUtil.getDbControl();
			DerivedBioAssay dbas = DerivedBioAssay.getById(dc, id);
			write_item(0, dbas);
			write("--Load derived bioassay OK");
		}
		catch (Throwable ex)
		{
			write("--Load derived bioassay FAILED");
			ex.printStackTrace();
			ok = false;
		}
		finally
		{
			if (dc != null) dc.close();
		}
	}

	static boolean test_list(int physicalBioAssayId, int expectedResults)
	{
		DbControl dc = null;
		try
		{
			dc = TestUtil.getDbControl();
			ItemQuery<DerivedBioAssay> query = null;
			if (physicalBioAssayId == 0)
			{
				query = DerivedBioAssay.getQuery();
			}
			else
			{
				PhysicalBioAssay bioAssay = PhysicalBioAssay.getById(dc, physicalBioAssayId);
				query = bioAssay.getRootDerivedBioAssays();
			}
			ItemResultList<DerivedBioAssay> l = query.list(dc);
			for (int i = 0; i<l.size(); i++)
			{
				write_item(i, l.get(i));
			}
			if (expectedResults >= 0 && expectedResults != l.size())
			{
				throw new BaseException("Expected "+expectedResults+" results, not "+l.size());
			}
			write("--List derived bioassays OK ("+l.size()+")");
		}
		catch (Throwable ex)
		{
			write("--List derived bioassays FAILED");
			ex.printStackTrace();
			ok = false;
			return false;
		}
		finally
		{
			if (dc != null) dc.close();
		}
		return true;
	}

	static void test_delete(int id)
	{
		if (id == 0) return;
		DbControl dc = null;
		try
		{
			dc = TestUtil.getDbControl();
			DerivedBioAssay dbas = DerivedBioAssay.getById(dc, id);
			dc.deleteItem(dbas);
			Set<ItemProxy> using = dbas.getUsingItems();
			if (using.size() > 0) 
			{
			   throw new BaseException(dbas + " is used by " + using);
			}
			dc.commit();
			write("--Delete derived bioassay OK");
		}
		catch (Throwable ex)
		{
			write("--Delete derived bioassay FAILED");
			ex.printStackTrace();
			ok = false;
		}
		finally
		{
			if (dc != null) dc.close();
		}
	}

	static void test_delete_all()
	{
		DbControl dc = null;
		try
		{
			dc = TestUtil.getDbControl();
			ItemResultList<DerivedBioAssay> l = DerivedBioAssay.getQuery().list(dc);
			int numDeleted = Trashcan.delete(dc.getSessionControl(), l, true, null);
			if (numDeleted != l.size()) throw new BaseException("Could not delete all items: " + numDeleted + " of " + l.size());
			dc.commit();
			write("--Delete all derived bioassays OK ("+l.size()+")");
		}
		catch (Throwable ex)
		{
			write("--Delete all derived bioassays FAILED");
			ex.printStackTrace();
			ok = false;
		}
		finally
		{
			if (dc != null) dc.close();
		}
	}

	
	static void write_header()
	{
		if (!TestUtil.getSilent())
		{
			write("   \tID \tName      \tType\tDescription\tParent\tPhysical bioassay\tProtocol\tHardware\tSoftware");
			write("-- \t-- \t--------- \t----\t-----------\t------\t----------------\t---------\t--------\t--------");
		}
	}
	static void write_item(int i, DerivedBioAssay dbas)
		throws BaseException
	{
		if (!TestUtil.getSilent()) 
		{
			System.out.println(i+":\t"+dbas.getId()+"\t"+dbas.getName()+"\t"+dbas.getItemSubtype()+"\t"+dbas.getDescription()+
				"\t"+dbas.getParent()+"\t"+dbas.getPhysicalBioAssay()+
				"\t"+dbas.getProtocol()+"\t"+dbas.getHardware()+"\t"+dbas.getSoftware());
		}
	}
	
	static void write(String message)
	{
		System.out.println(message);
	}
	
	static void test_list_children(int id, int expectedResults)
	{
		if (id == 0) return;
		DbControl dc = null;
		try
		{
			dc = TestUtil.getDbControl();
			DerivedBioAssay parent = DerivedBioAssay.getById(dc, id);
			ItemResultList<DerivedBioAssay> l = parent.getChildren().list(dc);
			for (int i = 0; i<l.size(); i++)
			{
				write_item(i, l.get(i));
			}
			if (expectedResults >= 0 && expectedResults != l.size())
			{
				throw new BaseException("Expected "+expectedResults+" results, not "+l.size());
			}
			write("--List child derived bioassays OK ("+l.size()+")");
		}
		catch (Throwable ex)
		{
			write("--List child derived bioassays FAILED");
			ex.printStackTrace();
			ok = false;
		}
		finally
		{
			if (dc != null) dc.close();
		}
	}

}
