/**
	$Id: Run.java 4806 2009-03-10 14:02:46Z nicklas $

	Copyright (C) 2006, 2007 Nicklas Nordborg

	This file is part of BASE - BioArray Software Environment.
	Available at http://base.thep.lu.se/

	BASE is free software; you can redistribute it and/or modify it
	under the terms of the GNU General Public License as published by
	the Free Software Foundation; either version 3 of the License, or
	(at your option) any later version.

	BASE is distributed in the hope that it will be useful, but
	WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
	General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with BASE. If not, see <http://www.gnu.org/licenses/>.
*/
package net.sf.basedb.test.performance;

import net.sf.basedb.test.CmdLine;
import net.sf.basedb.test.TestUtil;
import net.sf.basedb.util.Values;

/**

	@author nicklas
	@version 2.0
	@base.modified $Date: 2009-03-10 15:02:46 +0100 (Tue, 10 Mar 2009) $
*/
public class Run
{

	public static void main(String[] args)
	{
		
		CmdLine cmdLine = new CmdLine(args);
		String cmd = cmdLine.getCmd();

		String user = cmdLine.getOption("-u", "root");
		String password = cmdLine.getOption("-p", "root");
		boolean execute = cmdLine.hasOption("-x");
		int numIterations = Values.getInt(cmdLine.getOption("-i"), 1);
		String sourceNamePrefix = cmdLine.getOption("-s", null);
		String childNamePrefix = cmdLine.getOption("-c", null);

		int tests = 0;
		TestUtil.begin(false, true);
		boolean ok = PrepareTest.run(user, password);
		if ("raw".equals(cmd))
		{
			ok = RawDataTest.run(user, password, childNamePrefix, numIterations, execute);
			tests++;
		}
		else if ("root".equals(cmd))
		{
			int numSourcesStart = Values.getInt(cmdLine.getOption("-mi"), 1);
			int numSourcesEnd = Values.getInt(cmdLine.getOption("-mx"), 1);
			ok = RootTest.run(user, password, sourceNamePrefix, childNamePrefix, numIterations, 
					numSourcesStart, numSourcesEnd, execute);
			tests++;
		}
		else if ("filter".equals(cmd))
		{
			ok = FilterTest.run(user, password, sourceNamePrefix, childNamePrefix, execute);
			tests++;
		}
		else if ("lowess".equals(cmd))
		{
			ok = LowessTest.run(user, password, sourceNamePrefix, childNamePrefix, execute);
			tests++;
		}
		else if ("export".equals(cmd))
		{
			ok = ExportTest.run(user, password, sourceNamePrefix, execute);
			tests++;
		}
		else if ("clean".equals(cmd))
		{
			ok = PrepareTest.cleanUp(user, password);
			tests++;
		}
		TestUtil.stop();
		
		if (tests == 0)
		{
			showUsage();
		}
	}
	
	
	public static void showUsage()
	{
		TestUtil.write(
			"test.sh performance [-option value] <cmd>\n" +
			"    cmd = the test to run; raw, root, filter, lowess, export, clean\n\n" +
			"global options:\n" + 
			"     -u = username (default=root)\n" +
			"     -p = password (default=root)\n" +
			"     -x = if present, jobs are executed immediately\n" +
			"     -s = Source prefix; default varies by cmd\n" +
			"     -c = Child prefix; default varies by cmd\n" +
			"raw:\n" +
			"     -i = Number of raw data sets to import\n"+
			"     -c = Name prefix for created raw bioassays (default=Raw)\n"+
			"root:\n"+
			"     -s = Only use raw bioassays that matches the prefix (default=null)\n" +
			"     -c = Prefix to use for created bioassay sets (default=Root)\n" +
			"     -i = Number of root:s to to create for each iteration (default=1)\n"+
			"    -mi = Number of raw bioassays to use in the first iteration (default=1)\n"+
			"    -mx = Number of raw bioassay to use in the last iteration (default=1)\n"+
			"          Each iteration doubles the number of raw bioassays\n"+
			"filter:\n"+
			"     -s = Filter all bioassay sets that matches the prefix (default=Root)\n"+
			"     -c = Prefix to use for created bioassay sets (default=Filtered)\n"+
			"lowess:\n" +
			"     -s = Normalize all bioassay sets that matches the prefix (default=Filtered)\n"+
			"     -c = Prefix to use for created bioassay sets (default=Lowess)\n"+
			"export:\n"+
			"     -s = Export all bioassay sets that matches the prefix (no default)\n"+
			"\n"
		);
		
	}
	
}
