/**
	$Id: Run.java 4628 2008-11-05 15:43:03Z martin $

	Copyright (C) 2006 Nicklas Nordborg

	This file is part of BASE - BioArray Software Environment.
	Available at http://base.thep.lu.se/

	BASE is free software; you can redistribute it and/or modify it
	under the terms of the GNU General Public License as published by
	the Free Software Foundation; either version 3 of the License, or
	(at your option) any later version.

	BASE is distributed in the hope that it will be useful, but
	WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
	General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with BASE. If not, see <http://www.gnu.org/licenses/>.
*/
package net.sf.basedb.test.roles;

import java.util.Arrays;
import java.util.HashSet;
import java.util.Set;

import net.sf.basedb.test.TestUtil;

/**

	@author nicklas
	@version 2.0
	@base.modified $Date: 2008-11-05 16:43:03 +0100 (Wed, 05 Nov 2008) $
*/
public class Run
{

	public static void main(String[] args)
	{
		
		Set<String> cmds = new HashSet<String>(Arrays.asList(args));
		boolean all = cmds.contains("all");
		boolean logMemoryUsage = cmds.contains("-m");
		boolean useBatchImporters = cmds.contains("-b");
		
		int tests = 0;
		boolean ok = true;
		if (ok && (all || cmds.contains("root")))
		{
			TestUtil.begin(logMemoryUsage, true);
			ok = RootTest.run();
			tests++;
		}
		if (ok && (all || cmds.contains("admin")))
		{
			TestUtil.begin(logMemoryUsage, true);
			ok = AdminTest.run();
			tests++;
		}
		if (ok && (all || cmds.contains("power")))
		{
			TestUtil.begin(logMemoryUsage, true);
			ok = PowerUserTest.run(useBatchImporters);
			tests++;
		}
		if (ok && (all || cmds.contains("user")))
		{
			TestUtil.begin(logMemoryUsage, true);
			ok = UserTest.run(useBatchImporters);
			tests++;
		}
		if (ok && (all || cmds.contains("guest")))
		{
			TestUtil.begin(logMemoryUsage, true);
			ok = GuestTest.run();
			tests++;
		}
		TestUtil.stop();
		
		if (tests == 0)
		{
			showUsage();
		}
	}
	
	
	public static void showUsage()
	{
		TestUtil.write(
			"test.sh roles [OPTION] <cmds>\n" + 
			"     -m = log memory usage to file memory.log\n" +
			"	  -b = use the batch importers to insert items into BASE.\n" +	
			"     cmds = list of tests to run; all, root, admin, power, user, guest\n\n");
		
	}
	
}
