<%-- $Id: tree.jsp 6025 2012-03-23 07:45:15Z nicklas $
	------------------------------------------------------------------
	Copyright (C) 2009 Nicklas Nordborg

	This file is part of BASE - BioArray Software Environment.
	Available at http://base.thep.lu.se/

	BASE is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 3
	of the License, or (at your option) any later version.

	BASE is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with BASE. If not, see <http://www.gnu.org/licenses/>.
	------------------------------------------------------------------
--%>
<%@ page pageEncoding="UTF-8" session="false"
	import="net.sf.basedb.core.SessionControl"
	import="net.sf.basedb.core.DbControl"
	import="net.sf.basedb.core.Project"
	import="net.sf.basedb.core.Item"
	import="net.sf.basedb.core.BasicItem"
	import="net.sf.basedb.core.Annotation"
	import="net.sf.basedb.core.AnnotationType"
	import="net.sf.basedb.core.ItemSubtype"
	import="net.sf.basedb.core.Nameable"
	import="net.sf.basedb.core.Subtypable"
	import="net.sf.basedb.core.ItemContext"
	import="net.sf.basedb.core.Permission"
	import="net.sf.basedb.clients.web.Base"
	import="net.sf.basedb.clients.web.util.HTML"
	import="net.sf.basedb.util.Values"
	import="net.sf.basedb.util.overview.GenericOverview"
	import="net.sf.basedb.util.overview.OverviewUtil"
	import="net.sf.basedb.util.overview.Node"
	import="net.sf.basedb.util.overview.ValidationOptions"
	import="java.util.List"
%>
<%@ taglib prefix="base" uri="/WEB-INF/base.tld" %>
<%!
String generateSubTree(DbControl dc, Node startNode, String ID, boolean showFailures)
{
	StringBuilder sb = new StringBuilder();
	List<Node> children = startNode.getChildren();
	if (children == null) return "";
	for (Node child : children)
	{
		int errors = child.getNumErrors();
		int warnings = child.getNumWarnings();
		int childErrors = child.getChildErrors();
		int childWarnings = child.getChildWarnings();

		String folderIcon = child.getNodeType() == Node.Type.FOLDER ? "Folder" : "Item";
		String tooltip = "";
		if (child.getItemType() == Item.ANNOTATIONTYPE)
		{
			if (startNode.getItemType() == Item.PROTOCOL)
			{
				folderIcon = "ProtocolParameter";
			}
			else
			{
				try
				{
					AnnotationType at = (AnnotationType)child.getItem(dc);
					if (at.isProtocolParameter()) folderIcon = "ProtocolParameter";
				}
				catch (Throwable t)
				{}
			}
		}
		else if (child.getItemType() == Item.ANNOTATION)
		{
			folderIcon = "Annotation";
			try
			{
				AnnotationType at = ((Annotation)child.getItem(dc)).getAnnotationType();
				if (at.isProtocolParameter()) folderIcon = "ProtocolParameter";
			}
			catch (Throwable t)
			{}
		}
		
		if (showFailures)
		{
			if (errors > 0)
			{
				folderIcon += "Error";
			}
			else if (warnings > 0)
			{
				folderIcon += "Warning";
			}
			else if (childErrors > 0)
			{
				folderIcon += "ChildError";
			}
			else if (childWarnings > 0)
			{
				folderIcon += "ChildWarning";
			}			
			if (warnings > 0 && errors > 0)
			{
				tooltip += errors + " error(s); " + warnings + " warning(s) on this item";
			}
			else if (errors > 0)
			{
				tooltip += errors + " error(s) on this item";
			}
			else if (warnings > 0)
			{
				tooltip += warnings + " warning(s) on this item";
			}
			if (childErrors > 0 && childWarnings > 0)
			{
				if (tooltip.length() > 0) tooltip += "; ";
				tooltip += childErrors + " error(s); " + childWarnings + " warning(s) on child items";
			}
			else if (childErrors > 0)
			{
				if (tooltip.length() > 0) tooltip += "; ";
				tooltip += childErrors + " error(s) on child items";
			}
			else if (childWarnings > 0)
			{
				if (tooltip.length() > 0) tooltip += "; ";
				tooltip += childWarnings + " warning(s) on child items";
			}
		}
		int numChildren = child.getChildren() == null ? 0 : child.getChildren().size();
		String subtitle = null;
		if (child.getNodeType() == Node.Type.FOLDER)
		{
			subtitle = child.getChildren() == null ? "0" : Integer.toString(child.getChildren().size());
		}
		else if (child.getItem() instanceof Subtypable)
		{
			try
			{
				ItemSubtype subtype = ((Subtypable)child.getItem(dc)).getItemSubtype();
				if (subtype != null) subtitle = subtype.getName();
			}
			catch (Throwable t)
			{}
		}
		
		if (!child.isChildrenLoaded())
		{
			sb.append("var node").append(child.hashCode()).append(" = JoustMenu.addLazyChildItem(").append("node"+startNode.hashCode());
			sb.append(",'").append(folderIcon).append("'");
			sb.append(",'").append(HTML.javaScriptEncode(child.getTitle()));
			sb.append(subtitle != null ? " <span class=\"subtitle\">(" + subtitle + ")</span>" : "").append("',");
			sb.append("'showInfo(\"").append(child.getId()).append("\")','").append(HTML.javaScriptEncode(tooltip)).append("', '").append(child.getId()).append("',");
			sb.append("'lazyInitSubNode(\"").append(child.getId()).append("\")')\n");
		}
		else
		{
			 sb.append("var node").append(child.hashCode()).append(" = JoustMenu.addChildItem(").append("node"+startNode.hashCode());
          sb.append(",'").append(folderIcon).append("'");
          sb.append(",'").append(HTML.javaScriptEncode(child.getTitle()));
          sb.append(subtitle != null ? " <span class=\"subtitle\">(" + subtitle + ")</span>" : "").append("',");
          sb.append("'showInfo(\"").append(child.getId()).append("\")','").append(HTML.javaScriptEncode(tooltip)).append("', '").append(child.getId()).append("')\n");
		}
		sb.append(generateSubTree(dc, child, ID, showFailures));
	}
	return sb.toString();
}
%>
<%
final SessionControl sc = Base.getExistingSessionControl(pageContext, true);
final String ID = sc.getId();
final float scale = Base.getScale(sc);
final DbControl dc = sc.newDbControl();
try
{
	GenericOverview overview = OverviewUtil.getCurrentOverview(sc);
	Boolean showFailures = Values.getBoolean(request.getParameter("show_failures"), false);
	Node rootNode = overview.getRootNode();
	Nameable rootItem = (Nameable)rootNode.getItem(dc);
	String subtitle = null;
	if (rootItem instanceof Subtypable)
	{
		try
		{
			ItemSubtype subtype = ((Subtypable)rootItem).getItemSubtype();
			if (subtype != null) subtitle = subtype.getName();
		}
		catch (Throwable t)
		{}
	}
	String rootIcon = "Home";
	if (showFailures)
	{
		if (rootNode.getNumErrors() > 0 || rootNode.getChildErrors() > 0)
		{
			rootIcon = "Error";
		}
		else if (rootNode.getNumWarnings() > 0 || rootNode.getChildWarnings() > 0)
		{
			rootIcon = "Warning";
		}
	}
	%>
	<base:page title="" type="iframe">
	<base:head scripts="newjoust.js,ajax.js,json2.js" styles="newjoust.css">
	<script language="JavaScript">
	var isInitialised = false;
	function initialise() 
	{
		IconStore.init();
		var path = getRoot()+'images/joust/';
		
		IconStore.addIcon('Warning', getRoot() + 'images/warning.png', 16, 16);
		IconStore.addIcon('Error', getRoot() + 'images/error.png', 16, 16);
		IconStore.addIcon('FolderWarning', path + 'folderwarning.gif', 18, 16);
		IconStore.addIcon('FolderWarningSelected', path + 'folderwarningselected.gif', 18, 16);
		IconStore.addIcon('FolderChildWarning', path + 'folderchildwarning.gif', 18, 16);
		IconStore.addIcon('FolderChildWarningSelected', path + 'folderchildwarningselected.gif', 18, 16);
		IconStore.addIcon('FolderError', path + 'foldererror.gif', 18, 16);
		IconStore.addIcon('FolderErrorSelected', path + 'foldererrorselected.gif', 18, 16);
		IconStore.addIcon('FolderChildError', path + 'folderchilderror.gif', 18, 16);
		IconStore.addIcon('FolderChildErrorSelected', path + 'folderchilderrorselected.gif', 18, 16);
		IconStore.addIcon('Item', path + 'item.gif', 18, 16);
		IconStore.addIcon('ItemSelected', path + 'itemselected.gif', 18, 16);
		IconStore.addIcon('ItemWarning', path + 'itemwarning.gif', 18, 16);
		IconStore.addIcon('ItemWarningSelected', path + 'itemwarningselected.gif', 18, 16);
		IconStore.addIcon('ItemChildWarning', path + 'itemchildwarning.gif', 18, 16);
		IconStore.addIcon('ItemChildWarningSelected', path + 'itemchildwarningselected.gif', 18, 16);
		IconStore.addIcon('ItemError', path + 'itemerror.gif', 18, 16);
		IconStore.addIcon('ItemErrorSelected', path + 'itemerrorselected.gif', 18, 16);
		IconStore.addIcon('ItemChildError', path + 'itemchilderror.gif', 18, 16);
		IconStore.addIcon('ItemChildErrorSelected', path + 'itemchilderrorselected.gif', 18, 16);
		IconStore.addIcon('ProtocolParameter', path + 'parameter.gif', 18, 16);
		IconStore.addIcon('ProtocolParameterSelected', path + 'parameterselected.gif', 18, 16);
		IconStore.addIcon('ProtocolParameterWarning', path + 'parameterwarning.gif', 18, 16);
		IconStore.addIcon('ProtocolParameterWarningSelected', path + 'parametererrorselected.gif', 18, 16);
		IconStore.addIcon('ProtocolParameterError', path + 'parametererror.gif', 18, 16);
		IconStore.addIcon('ProtocolParameterErrorSelected', path + 'parametererrorselected.gif', 18, 16);
		IconStore.addIcon('ProtocolParameterChildWarning', path + 'parameterchildwarning.gif', 18, 16);
		IconStore.addIcon('ProtocolParameterChildWarningSelected', path + 'parameterchilderrorselected.gif', 18, 16);
		IconStore.addIcon('ProtocolParameterChildError', path + 'parameterchilderror.gif', 18, 16);
		IconStore.addIcon('ProtocolParameterChildErrorSelected', path + 'parameterchilderrorselected.gif', 18, 16);
		IconStore.addIcon('Annotation', path + 'annotation.gif', 18, 16);
		IconStore.addIcon('AnnotationSelected', path + 'annotationselected.gif', 18, 16);
		IconStore.addIcon('AnnotationWarning', path + 'annotationwarning.gif', 18, 16);
		IconStore.addIcon('AnnotationWarningSelected', path + 'annotationwarningselected.gif', 18, 16);
		IconStore.addIcon('AnnotationError', path + 'annotationerror.gif', 18, 16);
		IconStore.addIcon('AnnotationErrorSelected', path + 'annotationerrorselected.gif', 18, 16);
		IconStore.addIcon('AnnotationChildWarning', path + 'annotationchildwarning.gif', 18, 16);
		IconStore.addIcon('AnnotationChildWarningSelected', path + 'annotationchildwarningselected.gif', 18, 16);
		IconStore.addIcon('AnnotationChildError', path + 'annotationchilderror.gif', 18, 16);
		IconStore.addIcon('AnnotatioChildnErrorSelected', path + 'annotationchilderrorselected.gif', 18, 16);
		var node<%=rootNode.hashCode()%> = JoustMenu.addMenuItem(-1, '<%=rootIcon%>', '<%=HTML.javaScriptEncode(rootItem.getName())%><%=subtitle != null ? " <span class=\"subtitle\">(" + subtitle + ")</span>" : ""%>', 'showInfo("<%=rootNode.getId()%>")', '', '<%=rootNode.getId()%>');
		<%=generateSubTree(dc, rootNode, ID, showFailures)%>
		JoustMenu.menuItems[node<%=rootNode.hashCode()%>].isOpen = true;
		JoustMenu.draw('joust');
		isInitialised = true;
	}

	function openAll()
	{
		JoustMenu.openAll();
	}

	function closeAll()
	{
		JoustMenu.closeAll();
	}
	
	function showInfo(nodeId)
	{
		parent.frames['info'].location.href = 'info.jsp?ID=<%=ID%>&nodeId='+nodeId+'&show_failures=<%=showFailures%>';
	}
	
	function selectNode(nodeId)
	{
		var node = JoustMenu.menuItems[nodeId];
		JoustMenu.select(node.index);
	}
	
	var loadedTime = new Date().getTime();
	function refreshIfOutdated()
	{
		var currentTime = new Date().getTime();
		if (currentTime - loadedTime > 2000)
		{
			location.reload();
		}
	}

	function lazyInitSubNode(nodeId)
	{	
		var parentMenu = JoustMenu.menuItems[nodeId];
		JoustMenu.changeCursorOnMenuItem(parentMenu.index, 'progress');		
		var request = Ajax.getXmlHttpRequest();
		var url = 'ajax.jsp?ID=<%=ID%>&cmd=GetSubNodes&node_id=' + nodeId;
		request.open("GET", url, false);
		request.send(null);
		
		var response = Ajax.parseJsonResponse(request.responseText);
		if (response.status != 'ok')
		{
			alert(response.message);
			return false;
		}

		var childNodes = response.childNodes;
		for (var i=0; i<childNodes.length; i++)
		{
			var node = childNodes[i];
			var id = node.id;
			JoustMenu.addLazyChildItem(
					parentMenu.index, node.folderIcon, Main.trimString(node.title), 'showInfo(\''+id+'\')', 
					'' , id, node.numChildren == '0' ? '' : 'lazyInitSubNode(\''+id+'\')');
		}
		JoustMenu.updateIconsAndText(parentMenu.index, true);
		JoustMenu.changeCursorOnMenuItem(parentMenu.index, 'pointer');
		return childNodes.length > 0;					
	}
	</script>
	<style>
	.subtitle 
	{
		font-size: 0.85em;
		color: #666666;
	}
	.selected .subtitle 
	{
		color: #E8E8E8;
	}
	</style>
	
	</base:head>
		<base:body onload="initialise()">
		<div id="main" class="joust absolutefull" onmousemove="parent.DragDrop.drag(event)" onmouseup="parent.DragDrop.endDrag(event)">
			<div id="joust" style="overflow: auto; height: 100%;">
			</div>
		</div>
		</base:body>
	</base:page>
	<%
}
finally
{
	if (dc != null) dc.close();
}
%>
