/* $Id: annotations.js 3679 2007-08-17 07:18:29Z jari $
	------------------------------------------------------------------
	Copyright (C) 2005 Nicklas Nordborg
	Copyright (C) 2006 Jari Häkkinen, Nicklas Nordborg

	This file is part of BASE - BioArray Software Environment.
	Available at http://base.thep.lu.se/

	BASE is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 3
	of the License, or (at your option) any later version.

	BASE is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with BASE. If not, see <http://www.gnu.org/licenses/>.
	------------------------------------------------------------------

	JavaScript functions for the Platforms
	/common/platforms/*

	@author Nicklas
	@version 2.5
*/

var Platforms = new PlatformsClass();
function PlatformsClass()
{
	this.loadDataFilesFrame = function(theFrame, ID, itemType, itemId, platformId, variantId, itemSubtypeId)
	{
		var url = getRoot()+'common/datafiles/select_files.jsp?ID='+ID;
		url += '&item_type='+itemType+'&item_id='+itemId;
		if (platformId != undefined && platformId != null) url += '&platform_id='+platformId;
		if (variantId != undefined && variantId != null) url += '&variant_id='+variantId;
		if (itemSubtypeId != undefined && itemSubtypeId != null) url += '&itemsubtype_id='+itemSubtypeId;
		theFrame.location.href = url;
	}


	this.getPlatformById = function(id)
	{
		return allPlatforms['ID'+id];
	}
	
	this.populateList = function(list, platformId, variantId)
	{
		// Check that variant is part of platform; if ignore the variant
		if (platformId && variantId)
		{
			var platform = this.getPlatformById(platformId);
			if (platform)
			{
				var variant = platform.getVariantById(variantId);
				if (!variant) variantId = null;
			}
		}
		for (var p = 0; p < allPlatforms.length; p++)
		{
			var platform = allPlatforms[p];
			var option = new Option(platform.name, platform.id, false, platformId == platform.id && !variantId);
			list[list.length] = option;
			option.className = 'selectoptionheader';
			for (var v = 0; v < platform.variants.length; v++)
			{
				var variant = platform.variants[v];
				var name = variant.name;
				var prefix = v == platform.variants.length-1 ? ' └ ' : ' ├ ';
				option = new Option(prefix + name, platform.id + ':' + variant.id, false, platformId == platform.id && variantId == variant.id);
				list[list.length] = option;
			}
		}
	}
	
	this.getSelectedPlatform = function(list)
	{
		if (list.length == 0 || list.selectedIndex < 0) return null;
		var pv = list[list.selectedIndex].value.split(':');
		return this.getPlatformById(pv[0]);
	}

	this.getSelectedVariant = function(list)
	{
		if (list.length == 0 || list.selectedIndex < 0) return null;
		var pv = list[list.selectedIndex].value.split(':');
		if (pv.length < 2) return null;
		var platform = this.getPlatformById(pv[0]);
		return platform.getVariantById(pv[1]);
	}
	
}

var allPlatforms = new Array();
function Platform(id, externalId, name, fileOnly, rawDataType)
{
	this.id = id;
	this.externalId = externalId;
	this.name = name;
	this.fileOnly = fileOnly;
	this.rawDataType = rawDataType;
	this.variants = new Array();
	
	allPlatforms[allPlatforms.length] = this;
	allPlatforms['ID'+id] = this;
	
	this.getVariantById = function(id)
	{
		return this.variants['ID'+id];
	}
}

function Variant(platform, id, externalId, name, fileOnly, rawDataType)
{
	this.platform = platform;
	this.id = id;
	this.externalId = externalId;
	this.name = name;
	this.fileOnly = fileOnly;
	this.rawDataType = rawDataType;
	
	platform.variants[platform.variants.length] = this;
	platform.variants['ID'+id] = this;
}




