/* $Id: tabcontrol.js 5904 2011-12-09 13:52:53Z nicklas $
	------------------------------------------------------------------
	Copyright (C) 2005 Nicklas Nordborg
	Copyright (C) 2006 Jari Häkkinen, Nicklas Nordborg

	This file is part of BASE - BioArray Software Environment.
	Available at http://base.thep.lu.se/

	BASE is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 3
	of the License, or (at your option) any later version.

	BASE is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with BASE. If not, see <http://www.gnu.org/licenses/>.
	------------------------------------------------------------------

	JavaScript functions for the TabControl taglib.

	@author Nicklas
	@version 2.0
*/

var TabControl = new TabControlClass();
function TabControlClass()
{

	/**
		Set the initial active tab. It the checkRememebered
		parameter is set it will look check the top or opener window for
		a remembered active tab.
	*/
	this.setInitialTab = function(tabControlId, tabId, checkRemembered, switchFunction)
	{
		if (checkRemembered)
		{
			var tabControl = document.getElementById(tabControlId);
			if (!tabControl.activeTab)
			{
				var theWindow = window.opener ? window.opener : window;
				var rememberedTabId = TabControl.getRememberedTabId(theWindow, tabControlId);
				if (rememberedTabId) tabId = rememberedTabId
			}
		}
		if (switchFunction)
		{
			switchFunction(tabControlId, tabId);
		}
		else
		{
			TabControl.setActiveTab(tabControlId, tabId);
		}

	}

	/*
		Switch to another tab.
		@param tabControlId The ID attribute of the main tabcontrol
		@param tabId The ID attribute of the tab to make active
		@return If the call resulted in a switch
	*/
	this.setActiveTab = function(tabControlId, tabId)
	{
		var tabControl = document.getElementById(tabControlId);
		var theWindow = window.opener ? window.opener : window;
	
		var realTabId = tabControlId+'.'+tabId;
		var tab = document.getElementById(realTabId);
		if (tabControl.activeTab == tab) return false;
		if (!TabControl.validateActiveTab(tabControlId)) return false;
	
		if (tabControl.activeTab != null)
		{
			Main.removeClass(tabControl.activeTab, 'active');
		}
		Main.addClass(tab, 'active');

		tabControl.activeTab = tab;
		if (tabControl.activeContent != null)
		{
			tabControl.activeContent.style.display = 'none';
		}
		var content = document.getElementById(realTabId+'.content');
		if (content != null)
		{
			content.style.display = 'block';
			tabControl.activeContent = content;
		}
		if (tab.activate) eval(tab.activate);
		TabControl.rememberActiveTab(theWindow, tabControlId, tabId);
		return true;
	}

	this.rememberActiveTab = function(window, tabControlId, tabId)
	{
		if (window.Main) window.Main.setPageValue(location.pathname, tabControlId, tabId);
	}
	
	this.getRememberedTabId = function(window, tabControlId)
	{
		if (!window.Main) return null;
		return window.Main.getPageValue(location.pathname, tabControlId);
	}
	
	this.getActiveTabId = function(tabControlId)
	{
		var tabControl = document.getElementById(tabControlId);
		var activeTab = tabControl.activeTab;
		if (activeTab)
		{
			return activeTab.id.substr(tabControlId.length + 1);
		}
	}
	
	this.isActive = function(tabControlId, tabId)
	{
		var realTabId = tabControlId+'.'+tabId;
		var tabControl = document.getElementById(tabControlId);
		var tab = document.getElementById(realTabId);
		return tabControl.activeTab == tab;
	}
	
	this.getActiveTabHelpId = function(tabControlId)
	{
		var tabControl = document.getElementById(tabControlId);
		var activeTab = tabControl.activeTab;
		if (activeTab)
		{
			return activeTab.helpId;
		}
	}

	/**
		Validates the active tab
		@param tabControlId The ID attribute of the main tabcontrol
	*/
	this.validateActiveTab = function(tabControlId)
	{
		var tabControl = document.getElementById(tabControlId);
		if (tabControl.activeTab != null && tabControl.activeTab.validate != null)
		{
			return eval(tabControl.activeTab.validate);
		}
		else
		{
			return true;
		}
	}

	/**
		Set the validation function and help id for a tab
		@param tabControlId The ID attribute of the main tabcontrol
		@param tabId The ID attribute of the tab
		@param validate The function that validates the tab (optional)
		@param activate The funcation that is executed when the tab is activated (optional)
		@param helpId The help id for the tab
	*/
	this.initTab = function(tabControlId, tabId, validate, activate, helpId)
	{
		var realTabId = tabControlId+'.'+tabId;
		var tab = document.getElementById(realTabId);
		if (tab != null) 
		{
			if (validate) tab.validate = validate;
			if (activate) tab.activate = activate;
			if (helpId) tab.helpId = helpId;
		}
	}

}



