<%-- $Id: analysis_tree.jsp 6011 2012-03-13 12:14:41Z nicklas $
	------------------------------------------------------------------
	Copyright (C) 2006 Jari Häkkinen, Nicklas Nordborg, Martin Svensson
	Copyright (C) 2007 Johan Enell, Jari Häkkinen, Nicklas Nordborg

	This file is part of BASE - BioArray Software Environment.
	Available at http://base.thep.lu.se/

	BASE is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 3
	of the License, or (at your option) any later version.

	BASE is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with BASE. If not, see <http://www.gnu.org/licenses/>.
	------------------------------------------------------------------

	@author Nicklas
	@version 2.0
--%>
<%@ page pageEncoding="UTF-8" session="false"
	import="net.sf.basedb.core.SessionControl"
	import="net.sf.basedb.core.DbControl"
	import="net.sf.basedb.core.Item"
	import="net.sf.basedb.core.ItemContext"
	import="net.sf.basedb.core.BasicItem"
	import="net.sf.basedb.core.Experiment"
	import="net.sf.basedb.core.BioAssaySet"
	import="net.sf.basedb.core.Transformation"
	import="net.sf.basedb.core.ExtraValue"
	import="net.sf.basedb.core.ExtraValueType"
	import="net.sf.basedb.core.AnnotationSet"
	import="net.sf.basedb.core.AnnotationType"
	import="net.sf.basedb.core.Annotation"
	import="net.sf.basedb.core.Job"
	import="net.sf.basedb.core.ItemQuery"
	import="net.sf.basedb.core.Nameable"
	import="net.sf.basedb.core.ItemResultIterator"
	import="net.sf.basedb.core.ItemResultList"
	import="net.sf.basedb.core.Permission"
	import="net.sf.basedb.core.PluginDefinition"
	import="net.sf.basedb.core.PermissionDeniedException"
	import="net.sf.basedb.core.query.Restrictions"
	import="net.sf.basedb.core.query.Expressions"
	import="net.sf.basedb.core.query.Orders"
	import="net.sf.basedb.core.query.Hql"
	import="net.sf.basedb.core.query.Restrictions"
	import="net.sf.basedb.core.plugin.GuiContext"
	import="net.sf.basedb.core.plugin.Plugin"
	import="net.sf.basedb.util.Tree"
	import="net.sf.basedb.util.Enumeration"
	import="net.sf.basedb.clients.web.Base"
	import="net.sf.basedb.clients.web.ModeInfo"
	import="net.sf.basedb.clients.web.util.HTML"
	import="net.sf.basedb.util.Values"
	import="net.sf.basedb.util.formatter.Formatter"
	import="net.sf.basedb.clients.web.formatter.FormatterFactory"
	import="net.sf.basedb.clients.web.extensions.ExtensionsControl"
	import="net.sf.basedb.clients.web.extensions.toolbar.ButtonAction"
	import="net.sf.basedb.clients.web.extensions.toolbar.ToolbarUtil"
	import="net.sf.basedb.clients.web.extensions.JspContext"
	import="net.sf.basedb.clients.web.extensions.renderer.PrefixSuffixRenderer"
	import="net.sf.basedb.util.extensions.ExtensionsInvoker"
	import="net.sf.basedb.util.extensions.Renderer"
	import="java.util.Date"
	import="java.util.List"
	import="java.util.ArrayList"
	import="java.util.LinkedList"
	import="java.util.Map"
	import="java.util.HashMap"
	import="java.util.Iterator"
	import="java.util.Collection"
%>
<%@ taglib prefix="base" uri="/WEB-INF/base.tld" %>
<%@ taglib prefix="tbl" uri="/WEB-INF/table.tld" %>
<%@ taglib prefix="ext" uri="/WEB-INF/extensions.tld" %>
<%!
	private static final Item itemType = Item.BIOASSAYSET;
	private static final GuiContext guiContext = new GuiContext(itemType, GuiContext.Type.LIST);
%>
<%!

private static void addToTree(Tree<BasicItem> tree, BioAssaySet bas)
{
	if (!tree.contains(bas))
	{
		Transformation transformation = bas.getTransformation();
		Tree.Entry<BasicItem> parent = tree.getEntry(transformation);
		if (parent == null) 
		{
			addToTree(tree, transformation);
			parent = tree.getEntry(transformation);
		}
		Tree.Entry<BasicItem> basEntry = parent.addChild(bas);
		for (ExtraValue xv : bas.getExtraValues().list(bas.getDbControl())) 
		{
			basEntry.addChild(xv);
		}
	}
}

private static void addToTree(Tree<BasicItem> tree, Transformation transformation)
{
	if (!tree.contains(transformation))
	{
		BioAssaySet source = transformation.getSource();
		Tree.Entry<BasicItem> parent = tree.getEntry(source);
		if (parent == null) 
		{
			addToTree(tree, source);
			parent = tree.getEntry(source);
		}
		parent.addChild(transformation);
	}
}

private static Tree<BasicItem> getAnalysisTree(DbControl dc, ItemQuery<BioAssaySet> bioAssaySetQuery, ItemQuery<Transformation> transformationQuery)
{
	Tree<BasicItem> tree = new Tree<BasicItem>(null);
	ItemResultList<BioAssaySet> allBioAssaySets = bioAssaySetQuery.list(dc);
	List<Integer> ids = new ArrayList<Integer>(allBioAssaySets.size());
	
	for (BioAssaySet bas : allBioAssaySets)
	{
		addToTree(tree, bas);
		ids.add(bas.getId());
	}
	
	transformationQuery.restrict(
		Restrictions.in(
			Hql.property("source.id"), 
			Expressions.parameter("bioAssaySets", ids)
		)
	);
	ItemResultList<Transformation> allTransformations = transformationQuery.list(dc);
	for (Transformation t: allTransformations)
	{
		addToTree(tree, t);
	}
	return tree;
}
String generateTree(Tree<BasicItem> tree, BasicItem root, BasicItem parentItem, Collection<String> closed)
{
	StringBuilder sb = new StringBuilder();
	Tree.Entry<BasicItem> parentEntry = tree.getEntry(parentItem);
	if (parentEntry == null) return "";
	
	if (root != null && root == parentItem)
	{
		appendNode(sb, root, null, true);
	}
	
	List<Tree.Entry<BasicItem>> children = parentEntry.getChildren();
	if (children != null)
	{
		for (Tree.Entry<BasicItem> child : children)
		{
			BasicItem node = child.getNode();
			String var = node.getType().name() + "_" + node.getId();
			appendNode(sb, node, parentItem, (closed == null || !closed.contains(var)) && child.getNumChildren() > 0);
			sb.append(generateTree(tree, root, node, closed));
		}
	}
	return sb.toString();
}

void appendNode(StringBuilder sb, BasicItem node, BasicItem parentItem, boolean open)
{
	int id = node.getId();
	String var = node.getType().name() + "_" + id;
	String name = "";
	
	Item nodeType = node.getType();
	String folderIcon = "";
	if (nodeType == Item.BIOASSAYSET)
	{
		folderIcon = "BioAssaySet";
		name = ((BioAssaySet)node).getName();
	} 
	else if (nodeType == Item.TRANSFORMATION)
	{
		Transformation t = (Transformation)node;
		folderIcon = "Transformation";
		try
		{
			if (t.getJob().getPluginDefinition().supports("net.sf.basedb.core.plugin.AnalysisFilterPlugin"))
			{
				folderIcon = "Filter";
			}
		}
		catch (Throwable tt)
		{}
		name = t.getName();
	} 
	else if (nodeType == Item.EXTRAVALUE) 
	{
		folderIcon = "ExtraValue";
		try
		{
			name = ((ExtraValue)node).getExtraValueType().getName();
		}
		catch (PermissionDeniedException ex)
		{
			name = ((ExtraValue)node).getValueType().toString();
		}
	}
	sb.append("var ").append(var);
	if (parentItem == null)
	{
		sb.append(" = JoustMenu.addMenuItem(-1");
	}
	else
	{
		String rootVar = parentItem.getType().name() + "_" + parentItem.getId();
		sb.append(" = JoustMenu.addChildItem(").append(rootVar); 
	}
	sb.append(",'").append(folderIcon).append("'");
	sb.append(",'").append(HTML.javaScriptEncode(name)).append("'");
	sb.append(", null, '', '").append(var).append("');\n");
	if (open)
	{
		sb.append("JoustMenu.menuItems[").append(var).append("].isOpen = true;\n");
	}
}
%>
<%
final int experimentId = Values.getInt(request.getParameter("experiment_id"));
final int bioAssaySetId = Values.getInt(request.getParameter("item_id"));
final int transformationId = Values.getInt(request.getParameter("transformation_id"));
final SessionControl sc = Base.getExistingSessionControl(pageContext, Permission.DENIED, itemType);
final String ID = sc.getId();
final String rootPath = request.getContextPath()+"/";
final ItemContext cc = Base.getAndSetCurrentContext(sc, itemType, null, null);
final ItemContext tc = sc.getCurrentContext(Item.TRANSFORMATION);
final ItemContext xvc = sc.getCurrentContext(Item.EXTRAVALUE);

final ModeInfo mode = ModeInfo.get(request.getParameter("mode"));
final String callback = request.getParameter("callback");
final String title = mode.generateTitle("bioassay set", "bioassay sets");
final DbControl dc = sc.newDbControl();
Tree<BasicItem> analysisTree = null;
ItemResultList<AnnotationType> annotationTypes = null;
try
{
	Formatter<Date> dateTimeFormatter = FormatterFactory.getDateTimeFormatter(sc);
	final ItemQuery<AnnotationType> annotationTypeQuery = Base.getAnnotationTypesQuery(itemType);
	final Experiment experiment = Experiment.getById(dc, experimentId);
	final BasicItem root = transformationId == 0 ? 
			(bioAssaySetId == 0 ? null : BioAssaySet.getById(dc, bioAssaySetId))
			: Transformation.getById(dc, transformationId);
	final boolean createPermission = experiment.hasPermission(Permission.USE);
	final boolean deletePermission = createPermission;
	final boolean writePermission = createPermission;

	final ItemQuery<Transformation> transformationQuery = experiment.getTransformations();
	transformationQuery.include(cc.getInclude());
	transformationQuery.setFirstResult(0);
	transformationQuery.setMaxResults(-1);
	Map<Plugin.MainType, Integer> pluginCount = PluginDefinition.countPlugins(dc, guiContext);
	annotationTypes = annotationTypeQuery.list(dc);
	try
	{
		final ItemQuery<BioAssaySet> query = Base.getConfiguredQuery(dc, cc, true, experiment.getBioAssaySets(), mode);
		query.setFirstResult(0);
		query.setMaxResults(-1);
		analysisTree = getAnalysisTree(dc, query, transformationQuery);
	}
	catch (Throwable t)
	{
		cc.setMessage(t.getMessage());
		t.printStackTrace();
	}
	// Contains the ID:s of the bioassaysets that are closed in the tree
	Collection<String> closed = (Collection<String>)cc.getObject("closed");
	int numListed = 0;
	
	JspContext jspContext = ExtensionsControl.createContext(dc, pageContext, 
		guiContext, root == null ? experiment : root);
	ExtensionsInvoker toolsInvoker = ExtensionsControl.useExtensions(jspContext, 
			"net.sf.basedb.clients.web.bioassayset.list.tools");
	ExtensionsInvoker toolbarInvoker = ToolbarUtil.useExtensions(jspContext);
	%>
	<base:page type="include">
	<base:body>
		<ext:scripts context="<%=jspContext%>" />
		<ext:stylesheets context="<%=jspContext%>" />
		<script language="JavaScript">
		var submitPage = '<%=transformationId != 0 ? "../bioassaysets/index.jsp" : "index.jsp"%>';
		var formId = 'bioAssaySets';
		function newItem()
		{
			Main.openPopup('index.jsp?ID=<%=ID%>&cmd=NewItem&experiment_id=<%=experimentId%>', 'NewBioAssaySet', 750, 500);
		}
		function editBioAssaySet(itemId)
		{
			Main.viewOrEditItem('<%=ID%>', '<%=itemType.name()%>', itemId, true);
		}
		function viewBioAssaySet(itemId)
		{
			Main.viewOrEditItem('<%=ID%>', '<%=itemType.name()%>', itemId, false);
		}
		function bioAssaySetOnClick(evt, itemId)
		{
			Table.itemOnClick(formId, evt, itemId, '<%=mode.getName()%>', viewBioAssaySet, editBioAssaySet, returnSelected);
		}
		function viewTransformation(itemId)
		{
			Main.viewOrEditItem('<%=ID%>', 'TRANSFORMATION', itemId, false);
		}
		function editTransformation(itemId)
		{
			Main.viewOrEditItem('<%=ID%>', 'TRANSFORMATION', itemId, true);
		}
		function transformationOnClick(evt, itemId)
		{
			Table.itemOnClick(formId, evt, itemId, '<%=mode.getName()%>', viewTransformation, editTransformation, returnSelected);
		}
		function viewExtraValue(itemId)
		{
			Main.viewOrEditItem('<%=ID%>', 'EXTRAVALUE', itemId, false);
		}
		function editExtraValue(itemId)
		{
			Main.viewOrEditItem('<%=ID%>', 'EXTRAVALUE', itemId, true);
		}
		function extraValueOnClick(evt, itemId)
		{
			Table.itemOnClick(formId, evt, itemId, '<%=mode.getName()%>', viewExtraValue, editExtraValue, returnSelected);
		}
		function deleteItems()
		{
			var frm = document.forms[formId];
			if (Forms.numChecked(frm) == 0)
			{
				alert('Please select at least one item in the list');
				return; 
			}
			if (Forms.numChecked(frm, /^X:/) > 0)
			{
				if (!confirm('Extra values are deleted immediately and cannot be restored. Do you want to continue?'))
				{
					return;
				}
			}
			frm.action = submitPage;
			frm.cmd.value = 'DeleteItems';
			frm.submit();
		}
		function restoreItems()
		{
			var frm = document.forms[formId];
			if (Forms.numChecked(frm) == 0)
			{
				alert('Please select at least one item in the list');
				return; 
			}
			frm.action = submitPage;
			frm.cmd.value = 'RestoreItems';
			frm.submit();
		}
		function deleteItemPermanently(itemType, itemId)
		{
			Main.deleteItemPermanently('<%=ID%>', true, itemType, itemId);
		}
		function configureColumns()
		{
			Table.configureColumns('<%=ID%>', formId, '<%=itemType.name()%>', '<%=(String)cc.getObject("defaultColumns")%>');
		}
		function runPlugin(cmd)
		{
			Table.submitToPopup(formId, cmd, 750, 500);
		}
		function filter(itemId)
		{
			Main.openPopup('../bioassaysets/index.jsp?ID=<%=ID%>&cmd=NewFilteredBioAssaySet&experiment_id=<%=experimentId%>&item_id='+itemId, 'FilterBioAssaySet', 750, 500);
		}
		function openPlotTool(itemId)
		{
			Main.openPopup('../plotter/index.jsp?ID=<%=ID%>&bioassayset_id='+itemId, 'Plotter', 1050, 700);
		}
		function openExperimentExplorer(itemId)
		{
			location.href = '../explorer/view/index.jsp?ID=<%=ID%>&bioassayset_id='+itemId;
		}
		function runAnalysisPlugin(itemId)
		{
			Main.openPopup('../bioassaysets/index.jsp?ID=<%=ID%>&cmd=RunAnalysisPlugin&experiment_id=<%=experimentId%>&item_id='+itemId, 'RunPlugin', 750, 500);
		}
		function runExportPlugin(itemId)
		{
			Main.openPopup('../bioassaysets/index.jsp?ID=<%=ID%>&cmd=ExportItem&experiment_id=<%=experimentId%>&item_id='+itemId, 'Export', 750, 500);
		}
		function copyJob(jobId)
		{
			var url = '../../../common/plugin/index.jsp?ID=<%=ID%>';
			url += '&cmd=CopyJob&job_id='+jobId;
			url += '&item_type=BIOASSAYSET&context_type=LIST'
			Main.openPopup(url, 'CopyJob', 750, 500);
		}
		function returnSelected()
		{
			Table.returnSelected(formId, <%=callback != null ? "window.opener."+callback : "null" %>);
			window.close();
		}
		function presetOnChange()
		{
			Table.presetOnChange('<%=ID%>', formId, '<%=itemType.name()%>', '<%=(String)cc.getObject("defaultColumns")%>');
		}
		
		function sendClosedWithAjax()
		{
			var request = Main.getAjaxRequest();
			if (request != null)
			{
				var frm = document.forms[formId];
				var url = '../bioassaysets/index.jsp?ID=<%=ID%>&cmd=SetClosedWithAjax';
				request.open("POST", url, true);
				var postdata = 'closed=' + frm.closed.value;
				request.setRequestHeader("Content-type", "application/x-www-form-urlencoded");
				request.setRequestHeader("Content-length", postdata.length);
				request.setRequestHeader("Connection", "close");
				request.send(postdata);
			}
		}
		
		function initTree()
		{
			var bigDir = getRoot()+'images/joust/big/';
			IconStore.init(bigDir, 18, 22);
			IconStore.addIcon('BioAssaySet', bigDir + 'bioassayset.gif', 18, 22);
			IconStore.addIcon('Transformation', bigDir + 'transformation.gif', 18, 22);
			IconStore.addIcon('Filter', bigDir + 'filter.gif', 18, 22);
			IconStore.addIcon('ExtraValue', bigDir + 'extravalue.gif', 18, 22);

			JoustMenu.toggle = function(menuItemIndex)
			{
				var menuItem = this.menuItems[menuItemIndex];
				if (!menuItem) return;
				// Switch the open/closed status and hide or show the children
				menuItem.isOpen = !menuItem.isOpen;
				if (menuItem.isOpen)
				{
					this.showChildren(menuItemIndex);
					var frm = document.forms[formId];
					frm.closed.value = frm.closed.value.replace(','+menuItem.externalId+',', ',');
				}
				else
				{
					this.hideChildren(menuItemIndex);
					var frm = document.forms[formId];
					frm.closed.value += menuItem.externalId+',';
				}
				sendClosedWithAjax();
				// alert(document.forms[formId].closed.value);
				this.updateIconsAndText(menuItemIndex);
			}
			
			JoustMenu.drawMenuItems = function(firstIndex, indentString)
			{
				var menuItem = this.menuItems[firstIndex];
				while (menuItem)
				{
					var html = menuItem.draw(indentString);
					var padIcon = IconStore.getIcon(menuItem.noOutlineIcon == true ? null : menuItem.nextItemIndex == -1 ? 'iconBlank' : 'iconLine');
					var padHtml = padIcon == null ? '' : padIcon.getImgTag();
					
					var menuDiv = document.getElementById('tree.'+menuItem.externalId);
					menuDiv.innerHTML = html;
					
					if (menuItem.firstChildIndex != -1)
					{
						this.drawMenuItems(menuItem.firstChildIndex, indentString+padHtml);
					}
					if (!menuItem.isOpen)
					{
						this.hideChildren(menuItem.index);
					}
					menuItem = this.menuItems[menuItem.nextItemIndex];
				}
				return '';
			}

			JoustMenu.hideChildren = function(menuItemIndex)
			{
				var menuItem = this.menuItems[menuItemIndex];
				if (menuItem)
				{
					var firstChildIndex = menuItem.firstChildIndex;
					var child = this.menuItems[firstChildIndex];
					while (child)
					{
						var e = document.getElementById('row.'+child.externalId);
						e.style.display = 'none';
						this.hideChildren(child.index);
						child = this.menuItems[child.nextItemIndex];
					}
				}
			}
			
			JoustMenu.showChildren = function(menuItemIndex)
			{
				var menuItem = this.menuItems[menuItemIndex];
				if (menuItem)
				{
					var firstChildIndex = menuItem.firstChildIndex;
					var child = this.menuItems[firstChildIndex];
					while (child)
					{
						var e = document.getElementById('row.'+child.externalId);
						e.style.display = Browser.isIE ? 'block' : 'table-row';
						if (child.isOpen)
						{
							this.showChildren(child.index);
						}
						child = this.menuItems[child.nextItemIndex];
					}
				}
			}

			<%=analysisTree == null ? "" : generateTree(analysisTree, root, root, closed)%>
			JoustMenu.draw('joust');
			
		}
		</script>
		<tbl:table 
			id="bioAssaySets" 
			columns="<%=cc.getSetting("columns")%>"
			sortby="<%=cc.getSortProperty()%>" 
			direction="<%=cc.getSortDirection()%>"
			action="<%=transformationId != 0 ? "../bioassaysets/index.jsp" : "index.jsp"%>"
			sc="<%=sc%>"
			item="<%=itemType%>"
			subclass="<%=root == null ? "fulltable" : "" %>"
			>
			<tbl:hidden 
				name="mode" 
				value="<%=mode.getName()%>" 
			/>
			<tbl:hidden 
				name="experiment_id" 
				value="<%=String.valueOf(experimentId)%>" 
			/>
			<tbl:hidden 
				name="item_id" 
				value="<%=String.valueOf(bioAssaySetId)%>"
				skip="<%=bioAssaySetId == 0%>"
			/>
			<tbl:hidden 
				name="transformation_id" 
				value="<%=String.valueOf(transformationId)%>"
				skip="<%=transformationId == 0%>"
			/>
			<tbl:hidden 
				name="callback" 
				value="<%=callback%>" 
				skip="<%=callback == null%>" 
			/>
			<tbl:hidden
				name="closed"
				value="<%=closed == null ? "," : ","+Values.getString(closed, ",", true)+","%>"
			/>
			<tbl:columndef 
				id="name"
				property="name"
				datatype="string"
				title="Name"
				sortable="true" 
				filterable="true"
				exportable="true"
				show="always" 
			/>
			<tbl:columndef 
				id="spots"
				property="numSpots"
				datatype="int"
				title="Spots/Values in db"
				sortable="true" 
				filterable="true"
				exportable="true"
			/>
			<tbl:columndef 
				id="reporters"
				property="numReporters"
				datatype="int"
				title="Reporters in db"
				sortable="true" 
				filterable="true"
				exportable="true"
			/>
			<tbl:columndef 
				id="fileSpots"
				property="numFileSpots"
				datatype="int"
				title="Spots/Values in file"
				sortable="true" 
				filterable="true"
				exportable="true"
			/>
			<tbl:columndef 
				id="fileReporters"
				property="numFileReporters"
				datatype="int"
				title="Reporters in file"
				sortable="true" 
				filterable="true"
				exportable="true"
			/>
			<tbl:columndef 
				id="date"
				property="transformation.job.ended"
				datatype="string"
				title="Date"
				sortable="true" 
				filterable="true"
				exportable="true"
			/>
			<tbl:columndef 
				id="plugin"
				property="transformation.job.pluginDefinition.name"
				datatype="string"
				title="Plugin"
				sortable="true" 
				filterable="true"
				exportable="true"
			/>
			<tbl:columndef 
				id="description"
				property="description"
				datatype="string"
				title="Description" 
				sortable="true" 
				filterable="true" 
				exportable="true"
			/>
			<%
			for (AnnotationType at : annotationTypes)
			{
				Enumeration<String, String> annotationEnum = null;
				Formatter formatter = FormatterFactory.getTypeFormatter(sc, at.getValueType());
				if (at.isEnumeration())
				{
					annotationEnum = new Enumeration<String, String>();
					List<?> values = at.getValues();
					for (Object value : values)
					{
						String encoded = formatter.format(value);
						annotationEnum.add(encoded, encoded);
					}
				}
				%>
				<tbl:columndef 
					id="<%="at"+at.getId()%>"
					title="<%=HTML.encodeTags(at.getName())+" [A]"%>" 
					property="<%="#"+at.getId()%>"
					annotation="true"
					datatype="<%=at.getValueType().getStringValue()%>"
					enumeration="<%=annotationEnum%>"
					smartenum="<%=at.getDisplayAsList() %>"
					sortable="false" 
					filterable="true" 
					exportable="true"
					formatter="<%=formatter%>"
					unit="<%=at.getDefaultUnit()%>"
				/>
				<%
			}
			%>
			<tbl:columndef 
				id="tools"
				title="Tools" 
			/>

			<div class="panelgroup">
			<tbl:toolbar
				visible="<%=mode.hasToolbar()%>"
				subclass="<%=root == null ? "bottomborder" : "topborder leftborder rightborder" %>"
				>
				<tbl:button 
					disabled="<%=!createPermission%>" 
					image="new.png" 
					onclick="newItem()" 
					title="New root bioassay set&hellip;" 
					tooltip="<%=createPermission ? "Create a new root bioassay set" : "You do not have permission to create bioassay sets"%>" 
					visible="<%=root == null && pluginCount.containsKey(Plugin.MainType.INTENSITY)%>"
				/>
				<tbl:button 
					disabled="<%=!deletePermission%>" 
					image="delete.png" 
					title="Delete"
					onclick="deleteItems()" 
					tooltip="<%=deletePermission ? "Delete the selected items" : "You do not have permission to delete items"%>" 
				/>
				<tbl:button 
					disabled="<%=!writePermission%>" 
					image="restore.png" 
					onclick="restoreItems()" 
					title="Restore"
					tooltip="<%=writePermission ? "Restore the selected (deleted) items" : "You do not have permission to restore items"%>" 
				/>
				<tbl:button 
					image="columns.png" 
					onclick="configureColumns()" 
					title="Columns&hellip;" 
					tooltip="Show, hide and re-order columns" 
				/>
				<tbl:button 
					image="import.png" 
					onclick="runPlugin('ImportItems')" 
					title="Import&hellip;" 
					tooltip="Import data" 
					visible="<%=pluginCount.containsKey(Plugin.MainType.IMPORT)%>"
				/>
				<tbl:button 
					image="export.png" 
					onclick="runPlugin('ExportItems')" 
					title="Export&hellip;" 
					tooltip="Export data" 
					visible="<%=pluginCount.containsKey(Plugin.MainType.EXPORT)%>"
				/>
				<tbl:button 
					image="runplugin.png" 
					onclick="runPlugin('RunListPlugin')" 
					title="Run plugin&hellip;" 
					tooltip="Run a plugin" 
					visible="<%=pluginCount.containsKey(Plugin.MainType.OTHER)%>"
				/>
				<ext:render extensions="<%=toolbarInvoker%>" context="<%=jspContext%>" 
					wrapper="<%=new PrefixSuffixRenderer(jspContext, "<td>", "</td>") %>"/>
			</tbl:toolbar>
			</div>
			<tbl:data style="<%=root == null ? "top: 1.75em;" : "" %>">
				<tbl:headers>
					<tbl:headerrow>
						<tbl:header colspan="3">
							<tbl:presetselector 
								onchange="presetOnChange()"
								style="border-right: 0px;"
							/>
						</tbl:header>
						<tbl:columnheaders />
					</tbl:headerrow>
					<tbl:headerrow>
						<tbl:header subclass="index" />
						<tbl:header 
							subclass="check" 
							visible="<%=mode.hasCheck()%>"
							><base:icon 
								image="check_uncheck.png" 
								tooltip="Check/uncheck all" 
								onclick="Forms.checkUncheck(document.forms[formId])" 
							/></tbl:header>
						<tbl:header 
							subclass="check" 
							visible="<%=mode.hasRadio()%>"
							/>
						<tbl:header 
							subclass="icons" 
							visible="<%=mode.hasIcons()%>"
							/>
						<tbl:propertyfilter />
					</tbl:headerrow>
				</tbl:headers>
				<tbl:rows>
					<%
					if (cc.getMessage() != null)
					{
						%>
						<tbl:panel clazz="messagepanel">
							<div class="messagecontainer error"><%=cc.getMessage()%></div>
						</tbl:panel>
						<%
						cc.setMessage(null);
					}
					int index = cc.getPage()*cc.getRowsPerPage();
					int selectedItemId = cc.getId();
					if (analysisTree != null)
					{						
						Iterator<Tree.Entry<BasicItem>> baas = analysisTree.entryIterator(root);
						while (baas.hasNext())
						{
							Tree.Entry<BasicItem> entry = baas.next();
							BasicItem item = entry.getNode();
							if (item != null)
							{
								int level = entry.getDepth() - 1;
								int itemId = item.getId();
								String joustId = item.getType().name() + "_" + itemId;
								String name = "";
								String description = "";
								String tooltip = mode.isSelectionMode() ?
										"Select this item" : "View this item" + (writePermission ? " (use CTRL, ALT or SHIFT to edit)" : "");
								String deletePermanently = "deleteItemPermanently('" + item.getType().name() + "', "+itemId+")";
								boolean removed = false;
								Transformation t = null;
								BioAssaySet bas = null;
								ExtraValue xv = null;
								PluginDefinition plugin = null;
								Job job = null;
								String prefix = "";
								ItemContext ccc = null;
								String view = "";
								String onClick = "";
								Item itemType = item.getType();
								if (itemType == Item.TRANSFORMATION)
								{
									t = (Transformation)item;
									removed = t.isRemoved();
									prefix = "T:";
									ccc = tc;
									onClick = "transformationOnClick";
									name = HTML.encodeTags(t.getName());
									description = HTML.encodeTags(t.getDescription());
									try
									{
										job = t.getJob();
										plugin = job.getPluginDefinition();
									}
									catch (Throwable ex)
									{}
								}
								else if (itemType == Item.BIOASSAYSET)
								{
									bas = (BioAssaySet)item;
									removed = bas.isRemoved();
									ccc = cc;
									onClick = "bioAssaySetOnClick";
									name = HTML.encodeTags(bas.getName());
									description = HTML.encodeTags(bas.getDescription());
								}
								else if (itemType == Item.EXTRAVALUE)
								{
									xv = (ExtraValue)item;
									prefix = "X:";
									removed = false;
									ccc = xvc;
									onClick = "extraValueOnClick";
									try
									{
										ExtraValueType xvType = xv.getExtraValueType();
										name = HTML.encodeTags(xvType.getName());
										description = HTML.encodeTags(xvType.getDescription());
									}
									catch (PermissionDeniedException ex)
									{
										name = xv.getValueType().toString();
										description = "";
									}
									try
									{
										job = xv.getJob();
										plugin = job.getPluginDefinition();
									}
									catch (Throwable ex)
									{}
								}
								
								index++;
								numListed++;

								%>
								<tbl:row
									id="<%="row."+joustId%>"
									>
									<tbl:header 
										clazz="index"
										><%=index%></tbl:header>
									<tbl:header 
										clazz="check" 
										visible="<%=mode.hasCheck()%>"
										><input 
												type="checkbox" 
												name="<%=prefix+itemId%>" 
												value="<%=itemId%>" 
												title="<%=name%>" 
												<%=ccc.getSelected().contains(itemId) ? "checked" : ""%>
											></tbl:header>
									<tbl:header 
										clazz="check" 
										visible="<%=mode.hasRadio()%>"
										><input 
												type="radio" 
												name="item_id" 
												value="<%=itemId%>" 
												title="<%=name%>" 
												<%=selectedItemId == itemId ? "checked" : ""%>
											></tbl:header>
									<tbl:header 
										clazz="icons" 
										visible="<%=mode.hasIcons()%>"
										><base:icon 
											image="deleted.png"
											onclick="<%=deletePermission ? deletePermanently : null%>"
											tooltip="This item has been scheduled for deletion" 
											visible="<%=removed%>"
										/>&nbsp;</tbl:header>
									
									<tbl:cell clazz="cell joustcell" column="name">
										<div id="tree.<%=joustId%>" class="link" 
											onclick="<%=onClick%>(<%=writePermission ? "event" : null%>, <%=itemId%>)" 
											title="<%=tooltip%>"><%=name%>
										</div>
									</tbl:cell>
									<%
									if (t != null)
									{
										%>
										<tbl:cell column="spots">&nbsp;</tbl:cell>
										<tbl:cell column="reporters">&nbsp;</tbl:cell>
										<tbl:cell column="fileSpots">&nbsp;</tbl:cell>
										<tbl:cell column="fileReporters">&nbsp;</tbl:cell>
										<tbl:cell column="tools">
											<nobr>
											<%
											if (createPermission && job != null && plugin != null && plugin.isInteractive())
											{
												%>
												<a href="javascript:copyJob(<%=job.getId()%>)" 
													title="Copy this transformation"><img 
													src="../../../images/copy.png" border="0"></a>
												<%
											}
											%>
											<ext:render extensions="<%=toolsInvoker%>" context="<%=jspContext%>" item="<%=item%>" />
											</nobr>
										</tbl:cell>
										<%
									}
									if (bas != null)
									{
										%>
										<tbl:cell column="spots"><%=bas.getNumSpots()%></tbl:cell>
										<tbl:cell column="reporters"><%=bas.getNumReporters()%></tbl:cell>
										<tbl:cell column="fileSpots"><%=bas.getNumFileSpots()%></tbl:cell>
										<tbl:cell column="fileReporters"><%=bas.getNumFileReporters()%></tbl:cell>
										<tbl:cell column="date">&nbsp;</tbl:cell>
										<tbl:cell column="plugin">&nbsp;</tbl:cell>
										<%
										AnnotationSet as = bas.isAnnotated() ? bas.getAnnotationSet() : null;
										if (as != null)
										{
											for (AnnotationType at : annotationTypes)
											{
												if (as.hasAnnotation(at))
												{
													Annotation a = as.getAnnotation(at);
													String suffix = a.getUnitSymbol(null);
													if (suffix != null) suffix = "&nbsp;" + suffix;
													%>
													<tbl:cell 
														column="<%="at"+at.getId()%>"
														><tbl:cellvalue 
															list="<%=a.getValues(null)%>"
															suffix="<%=suffix%>"
													/></tbl:cell>
													<%
												}
											}
										}
										%>
										<tbl:cell column="tools">
											<nobr>
											<%
											if (bas.getNumSpots() > 0)
											{
												%>
												<a href="javascript:openPlotTool(<%=itemId%>)" 
													title="A simple plot tool"><img 
													src="../../../images/plotter.png" border="0"></a>
												<a href="javascript:openExperimentExplorer(<%=itemId%>)"
													title="Experiment explorer"><img
													src="../../../images/explorer.png" border="0"></a>
												<%
											}
											%>
											<a href="javascript:runExportPlugin(<%=itemId%>)"
												title="Export data"><img 
												src="../../../images/export.png" border="0"></a>
												
											<%
											if (createPermission)
											{
												%>
												<a href="javascript:filter(<%=itemId%>)" 
													title="Create a filtered bioassay set"><img 
													src="../../../images/filter.png" border="0"></a>
												<a href="javascript:runAnalysisPlugin(<%=itemId%>)" title="Run an analysis plugin"><img 
													src="../../../images/runplugin.png" border="0"></a>
												<%
											}
											%>
											<ext:render extensions="<%=toolsInvoker%>" context="<%=jspContext%>" item="<%=item%>" />
											</nobr>
										</tbl:cell>
										<%
									}
									if (xv != null)
									{
										%>
										<tbl:cell column="spots"><%=xv.getNumValues()%></tbl:cell>
										<tbl:cell column="reporters">&nbsp;</tbl:cell>
										<tbl:cell column="fileSpots"><%=xv.getNumFileValues()%></tbl:cell>
										<tbl:cell column="fileReporters">&nbsp;</tbl:cell>
										<tbl:cell column="tools">
											<nobr>
											<%
											if (createPermission && job != null && plugin != null && plugin.isInteractive())
											{
												%>
												<a href="javascript:copyJob(<%=job.getId()%>)" 
													title="Copy this extra value"><img 
													src="../../../images/copy.png" border="0"></a>
												<%
											}
											%>
											<ext:render extensions="<%=toolsInvoker%>" context="<%=jspContext%>" item="<%=item%>" />														
											</nobr>
										</tbl:cell>
										<%
									}
									%>
									<tbl:cell column="date"><%=job == null ? "" : dateTimeFormatter.format(job.getEnded())%></tbl:cell>
									<tbl:cell column="plugin"><%=plugin == null ? "" : Base.getLinkedName(ID, plugin, false, true)%></tbl:cell>
									<tbl:cell column="description"><%=description%></tbl:cell>
								</tbl:row>
								<%
							}
						}
					}
					if (numListed == 0)
					{
						%>
						<tbl:panel clazz="messagepanel">
							<div class="messagecontainer note">
							No bioassay sets or transformations were found.
							</div>
						</tbl:panel>
						<%
					}
					%>
				</tbl:rows>
			</tbl:data>
		</tbl:table>

		<script language="JavaScript">
		initTree();
		</script>
	</base:body>
	</base:page>
	<%
}
finally
{
	if (dc != null) dc.close();
}
%>
